@@ -21,10 +21,14 @@
 Dive computer functionality.
 """
 
-import lxml.etree as et
 from datetime import datetime
+from functools import partial
+import lxml.etree as et
 import logging
 
+import kenozooid.component as kc
+import kenozooid.data as kd
+import kenozooid.flow as kf
 import kenozooid.uddf as ku
 
 log = logging.getLogger('kenozooid.dc')
@@ -43,7 +47,6 @@ def backup(drv_name, port, fout):
     """
     drv = _mem_dump(drv_name, port)
     data = drv.dump()
-    model = drv.version(data)
 
     _save_dives(drv, datetime.now(), data, fout)
 
@@ -97,7 +100,7 @@ def extract_dives(fin, fout):
 
 def _mem_dump(name, port=None):
     """
-    Find memory dump device driver.
+    Find data parser device driver.
 
     :Parameters:
      name
@@ -105,11 +108,11 @@ def _mem_dump(name, port=None):
      port
         Dive computer port.
     """
-    from kenozooid.driver import MemoryDump, find_driver
+    from kenozooid.driver import DataParser, find_driver
 
-    drv = find_driver(MemoryDump, name, port)
+    drv = find_driver(DataParser, name, port)
     if drv is None:
-        raise ValueError('Device driver {} does not support memory dump'
+        raise ValueError('Device driver {} does not support data parsing'
             .format(name))
     return drv
 
@@ -129,34 +132,84 @@ def _save_dives(drv, time, data, fout):
      fout
         Output file.
     """
-    import kenozooid.uddf as ku
-    
     model = drv.version(data)
+    dc_id = ku.gen_id(model)
     log.debug('dive computer version {}'.format(model))
 
-    dout = ku.create()
-
-    # store dive computer information
-    xp_owner = ku.XPath('//uddf:diver/uddf:owner')
-    dc = ku.create_dc_data(xp_owner(dout)[0], dc_model=model)
-    dc_id = dc.get('id')
-
-    # store raw data
-    ddn = ku.create_dump_data(dout, dc_id=dc_id, datetime=time, data=data)
-    dump = ku.dump_data(ddn)
-
     # convert raw data into dive data and store in output file
-    dnodes = drv.dives(dump)
-    _, rg = ku.create_node('uddf:profiledata/uddf:repetitiongroup',
-            parent=dout)
-    for n in dnodes:
-        *_, l = ku.create_node('uddf:informationafterdive' \
-                '/uddf:equipmentused/uddf:link', parent=n)
-        l.set('ref', dc_id)
-        rg.append(n)
+    bdata = kd.BinaryData(datetime=time, data=data)
     
-    ku.reorder(dout)
-    ku.save(dout, fout)
+    eq = ku.create_dc_data(dc_id, model)
+    dump = ku.create_dump_data(dc_id=dc_id, datetime=time, data=data)
 
+    p = kc.params(drv.__class__)
+    if 'gas' in p['data']:
+        log.debug('gas data pipeline')
+        with kf.buffer_open(2) as (f_g, f_d):
+
+            # store gas and dives in two separate files,
+            # then merge the data into one UDDF file
+            save = kf.sink(partial(ku.save_uddf, fout=fout))
+            m = kf.concat(2, partial(cat_gd, equipment=eq, dump=dump), save)
+            kf.send(
+                kf.pipe(drv.dives(bdata),
+                    kd.sort_dives,
+                    kd.uniq_dives),
+                kf.split(
+                    extract_gases(uniq_gases(kf.buffer(f_g, m))),
+                    create_dives(kf.buffer(f_d, m), equipment=(dc_id,))
+                )
+            )
+
+    else:
+        log.debug('simple data pipeline')
+        dives = kf.pipe(drv.dives(bdata),
+                kd.sort_dives,
+                kd.uniq_dives,
+                partial(ku.create_dives, equipment=(dc_id,)))
+        ku.save_uddf(ku.create_uddf(equipment=eq, dives=dives, dump=dump), fout)
+
+#
+# fixme: put the functions/coroutines below into proper modules
+#
+@kf.coroutine
+def create_dives(tg, equipment=None):
+    while True:
+        dive = yield
+        n = ku.create_dive(dive, equipment=equipment)
+        tg.send(n)
+
+@kf.coroutine
+def extract_gases(tc):
+    while True:
+        dive = yield
+        for s in dive.profile:
+            if s.gas is not None:
+                tc.send(s.gas)
+
+@kf.coroutine
+def uniq_gases(tc):
+    gases = set()
+    while True:
+        gas = yield
+        if gas not in gases:
+            gases.add(gas)
+            n = ku.create_gas(gas)
+            tc.send(n)
+
+def cat_gd(fg, fd, equipment=None, dump=None):
+    xfg = xfd = None
+    fg.seek(0)
+    fd.seek(0)
+    if len(fg.read(1)) > 0:
+        xfg = ku.xml_file_copy(fg)
+    if len(fd.read(1)) > 0:
+        xfd = ku.xml_file_copy(fd)
+    fg.seek(0)
+    fd.seek(0)
+    return ku.create_uddf(equipment=equipment,
+            dump=dump,
+            gases=xfg,
+            dives=xfd)
 
 # vim: sw=4:et:ai
