@@ -27,21 +27,25 @@ protocol can be found at address
 
 """
 
-from collections import OrderedDict
 from datetime import datetime, timedelta
 from serial import Serial, SerialException
 from binascii import hexlify
 from functools import partial
+from operator import attrgetter
 import logging
 
 log = logging.getLogger('kenozooid.driver.ostc')
 
 import kenozooid.uddf as ku
 import kenozooid.component as kc
-from kenozooid.driver import DeviceDriver, Simulator, MemoryDump, DeviceError
+from kenozooid.driver import DeviceDriver, Simulator, DataParser, DeviceError
 from kenozooid.units import C2K
 from . import parser as ostc_parser
+import kenozooid.data as kd
 
+GAS_GETTERS = dict(
+    zip(range(1, 7), (attrgetter('gas{}_o2'.format(i), 'gas{}_he'.format(i))
+        for i in range(1, 7))))
 
 def pressure(depth):
     """
@@ -142,10 +146,10 @@ class OSTCSimulator(object):
 
 
 
-@kc.inject(MemoryDump, id='ostc')
-class OSTCMemoryDump(object):
+@kc.inject(DataParser, id='ostc', data=('gas',))
+class OSTCDataParser(object):
     """
-    OSTC dive computer memory dump.
+    OSTC dive computer data parser.
     """
     def dump(self):
         """
@@ -170,12 +174,9 @@ class OSTCMemoryDump(object):
             'uddf:decostop/@duration', 'uddf:decostop/@decodepth', 'uddf:decostop/@kind', \
             'uddf:depth', 'uddf:divetime', 'uddf:temperature',
                 
-        UDDF_SAMPLE = OrderedDict(zip(_f, _q))
+        ostc_data = ostc_parser.get_data(dump.data)
 
-        nodes = []
-        dive_data = ostc_parser.get_data(dump.data)
-
-        for h, p in ostc_parser.profiles(dive_data.profiles):
+        for h, p in ostc_parser.profiles(ostc_data.profiles):
             log.debug('header: {}'.format(hexlify(h)))
             log.debug('profile: {}'.format(hexlify(p)))
 
@@ -194,45 +195,64 @@ class OSTCMemoryDump(object):
             st -= duration
 
             try:
-                dn = ku.create_dive_data(datetime=st,
-                        depth=header.max_depth / 100.0,
-                        duration=duration.seconds,
-                        temp=C2K(header.min_temp / 10.0))
-
-                create_sample = partial(ku.create_dive_profile_sample, dn,
-                        queries=UDDF_SAMPLE)
-
-                # ostc start dive below zero, add (0, 0) waypoint to
-                # comply with uddf
-                create_sample(time=0, depth=0.0)
-
-                for i, sample in enumerate(dive_data, 1):
-                    temp = C2K(sample.temp) if sample.temp else None
-
-                    # deco info
-                    deco_time = sample.deco_time * 60.0 if sample.deco_depth else None
-                    deco_depth = sample.deco_depth if sample.deco_depth else None
-                    deco_kind = 'mandatory' if sample.deco_depth else None
-                    deco_alarm = sample.alarm in (2, 3)
-
-                    create_sample(time=(i * header.sampling),
-                            depth=sample.depth,
-                            alarm='deco' if deco_alarm else None,
-                            temp=temp,
-                            deco_time=deco_time,
-                            deco_depth=deco_depth,
-                            deco_kind=deco_kind)
-
-                create_sample(time=(i + 2) * header.sampling, depth=0.0)
-
-                yield dn
-
+                profile = list(self._get_profile(header, dive_data))
+                yield kd.Dive(datetime=st,
+                    depth=header.max_depth / 100.0,
+                    duration=duration.seconds,
+                    temp=C2K(header.min_temp / 10.0),
+                    profile=profile)
             except ValueError as ex:
                 log.error('invalid dive {0.year:>02d}-{0.month:>02d}-{0.day:>02d}' \
                     ' {0.hour:>02d}:{0.minute:>02d}' \
                     ' max depth={0.max_depth}'.format(header))
 
 
+    def _get_profile(self, header, dive_data):
+        """
+        Parse OSTC dive samples.
+        """
+        # ostc starts dive below at a depth, so add (0, 0) sample
+        yield kd.Sample(depth=0.0, time=0, gas=self._get_gas(header, header.gas))
+
+        for i, sample in enumerate(dive_data, 1):
+            temp = C2K(sample.temp) if sample.temp else None
+
+            gas = None
+            if sample.current_gas is not None:
+                gas = self._get_gas(header, sample.current_gas)
+            elif sample.gas_set_o2 is not None:
+                gas = kd.gas(sample.gas_set_o2, sample.gas_set_he)
+
+            # deco info
+            deco_time = sample.deco_time * 60.0 if sample.deco_depth else None
+            deco_depth = sample.deco_depth if sample.deco_depth else None
+            deco_alarm = sample.alarm in (2, 3)
+
+            yield kd.Sample(depth=sample.depth,
+                    time=(i * header.sampling),
+                    alarm=('deco',) if deco_alarm else None,
+                    temp=temp,
+                    deco_time=deco_time,
+                    deco_depth=deco_depth,
+                    gas=gas)
+
+        yield kd.Sample(depth=0.0, time=(i + 1) * header.sampling)
+
+
+    def _get_gas(self, header, gas_no):
+        """
+        Get gas information from OSTC dive header.
+
+        :Parameters:
+         header
+            Dive header information.
+         gas_no
+            Gas number to get (1-6).
+        """
+        o2, he = GAS_GETTERS[gas_no](header)
+        return kd.gas(o2=o2, he=he)
+
+
     def version(self, data):
         """
         Get OSTC model and version information from raw data.
@@ -243,7 +263,7 @@ class OSTCMemoryDump(object):
             model = 'OSTC N2'
         elif status.eeprom.serial > 300:
             model = 'OSTC Mk.2'
-        return '{} {}.{}'.format(model, status.ver1, status.ver2)
+        return '{} {}.{:02}'.format(model, status.ver1, status.ver2)
 
 
 # vim: sw=4:et:ai
