@@ -27,8 +27,9 @@ import lxml.etree as et
 import unittest
 
 import kenozooid.uddf as ku
+import kenozooid.data as kd
 import kenozooid.driver.ostc.parser as ostc_parser
-from kenozooid.driver.ostc import pressure, OSTCMemoryDump
+from kenozooid.driver.ostc import pressure, OSTCDataParser
 
 from . import data as od
 
@@ -46,83 +47,103 @@ class ConversionTestCase(unittest.TestCase):
 
 
 
-class UDDFTestCase(unittest.TestCase):
+class DataModelTestCase(unittest.TestCase):
     """
-    OSTC data to UDDF format conversion tests.
+    OSTC data to data model format conversion tests.
 
     :Attributes:
      dives
         List of dives parsed from DATA_OSTC.
     """
     def setUp(self):
-        super(UDDFTestCase, self).setUp()
-        DCDump = namedtuple('DCDump', 'datetime data')
+        super(DataModelTestCase, self).setUp()
 
-        data = ku._dump_decode(od.DATA_OSTC)
-        dump = DCDump(datetime.now(), data)
-
-        dc = OSTCMemoryDump()
+        dump = kd.BinaryData(datetime=datetime.now(), data=od.RAW_DATA_OSTC)
+        dc = OSTCDataParser()
         self.dives = list(dc.dives(dump))
 
 
     def test_conversion(self):
         """
-        Test basic OSTC data to UDDF conversion
+        Test basic OSTC data to data model conversion
         """
         # five dives
         self.assertEquals(5, len(self.dives))
 
         # 193 samples for first dive
         dive = self.dives[0]
-        data = list(ku.xp(dive, 'uddf:samples/uddf:waypoint'))
-        self.assertEquals(195, len(data))
+        samples = list(dive.profile)
+        self.assertEquals(195, len(samples))
 
-        t = ku.xp_first(dive, 'uddf:informationbeforedive/uddf:datetime/text()')
-        self.assertEquals('2009-01-31T23:08:41', t)
+        self.assertEquals(datetime(2009, 1, 31, 23, 8, 41), dive.datetime)
+        self.assertEquals(75.0, dive.depth)
+        self.assertEquals(1939, dive.duration)
+        self.assertEquals(300.65, dive.temp) # 27.45C
 
-        d = ku.xp_first(dive, 'uddf:informationafterdive/uddf:greatestdepth/text()')
-        self.assertEquals('75.0', d)
 
-        d = ku.xp_first(dive, 'uddf:informationafterdive/uddf:diveduration/text()')
-        self.assertEquals('1939', d)
+    def test_gas(self):
+        """
+        Test OSTC gas data to data model conversion
+        """
+        dump = kd.BinaryData(datetime=datetime.now(),
+                data=od.RAW_DATA_OSTC_MK2_196)
+        dc = OSTCDataParser()
+        dive = list(dc.dives(dump))[7]
+        samples = list(dive.profile)
+        self.assertEquals('tx1633', samples[0].gas.id)
+        self.assertEquals('tx1633', samples[1360].gas.id)
+        self.assertEquals('o2', samples[1372].gas.id)
+        self.assertEquals('ean47', samples[1491].gas.id)
 
-        t = ku.xp_first(dive, 'uddf:informationafterdive/uddf:lowesttemperature/text()')
-        self.assertEquals('300.6', t) # 27.45C
+
+    def test_gas_info(self):
+        """
+        Test OSTC gas fetching from header
+        """
+        dives = ostc_parser.profiles(od.RAW_DATA_OSTC_MK2_194)
+        header, profile = list(dives)[7]
+        header = ostc_parser.header(header)
+        dc = OSTCDataParser()
+
+        self.assertEquals('air', dc._get_gas(header, 1).id)
+        self.assertEquals('ean47', dc._get_gas(header, 2).id)
+        self.assertEquals('o2', dc._get_gas(header, 3).id)
+        self.assertEquals('tx1340', dc._get_gas(header, 4).id)
+        self.assertEquals('tx1737', dc._get_gas(header, 5).id)
+        self.assertEquals('tx1050', dc._get_gas(header, 6).id)
 
 
     def test_deco_alarm(self):
         """
-        Test OSTC deco alarm data to UDDF conversion
+        Test OSTC deco alarm data to data model conversion
         """
         dive = self.dives[3] # dive no 3 contains deco alarms
-        wps = list(ku.xp(dive, 'uddf:samples/uddf:waypoint'))
-        d1 = wps[587:594]
-        d2 = wps[743:749]
-        d3 = wps[972:976]
-        d4 = wps[1320:1343]
+        samples = list(dive.profile)
+        d1 = samples[587:594]
+        d2 = samples[743:749]
+        d3 = samples[972:976]
+        d4 = samples[1320:1343]
 
         # check if all deco waypoints have appropriate alarms
-        def alarms(wps):
-            return (ku.xp_first(w, 'uddf:alarm/text()') == 'deco' for w in wps)
+        def alarms(samples):
+            return (s.alarm == ('deco',) for s in samples)
 
         t1 = list(alarms(d1))
         t2 = list(alarms(d2))
         t3 = list(alarms(d3))
         t4 = list(alarms(d4))
-        self.assertTrue(all(t1), '{0}\n{1}'.format(t1, et.tostring(dive)))
-        self.assertTrue(all(t2), '{0}\n{1}'.format(t2, et.tostring(dive)))
-        self.assertTrue(all(t3), '{0}\n{1}'.format(t3, et.tostring(dive)))
-        self.assertTrue(all(t4), '{0}\n{1}'.format(t4, et.tostring(dive)))
+        self.assertTrue(all(t1), '{0}\n{1}'.format(t1, d1))
+        self.assertTrue(all(t2), '{0}\n{1}'.format(t2, d2))
+        self.assertTrue(all(t3), '{0}\n{1}'.format(t3, d3))
+        self.assertTrue(all(t4), '{0}\n{1}'.format(t4, d4))
 
 
     def test_deco(self):
         """
-        Test OSTC deco data to UDDF conversion
+        Test OSTC deco data to data model conversion
         """
         dive = self.dives[0]
-        sd = et.tostring(dive)
-
-        wps = list(ku.xp(dive, 'uddf:samples/uddf:waypoint'))[24:181:6]
+        samples = list(dive.profile)[24:181:6]
 
         # depth, time
         deco = ((3, 60), (12, 60), (9, 60), (12, 60), (12, 60), (3, 60), (6, 60),
@@ -131,18 +152,13 @@ class UDDFTestCase(unittest.TestCase):
                 (6, 120), (6, 60), (3, 300), (3, 180), (3, 120), (3, 60),)
 
         self.assertEquals(deco,
-            tuple((float(ku.xp_first(n, 'uddf:decostop/@decodepth')),
-                   float(ku.xp_first(n, 'uddf:decostop/@duration'))) for n in wps),
-            sd)
-
-        m = [ku.xp_first(n, 'uddf:decostop/@kind') == 'mandatory' for n in wps]
-        self.assertTrue(all(m), m)
+            tuple((float(s.deco_depth), float(s.deco_time)) for s in samples))
 
 
 
-class MemoryDumpTestCase(unittest.TestCase):
+class DataParserTestCase(unittest.TestCase):
     """
-    OSTC memory dump tests.
+    OSTC data parser tests.
 
     :Attributes:
      dump_data
@@ -153,7 +169,7 @@ class MemoryDumpTestCase(unittest.TestCase):
         """
         Test OSTC model and version parsing from raw data
         """
-        dc = OSTCMemoryDump()
+        dc = OSTCDataParser()
         ver = dc.version(od.RAW_DATA_OSTC)
         self.assertEquals('OSTC 1.26', ver)
 
@@ -162,7 +178,7 @@ class MemoryDumpTestCase(unittest.TestCase):
         """
         Test OSTC Mk.2 model and version parsing from raw data
         """
-        dc = OSTCMemoryDump()
+        dc = OSTCDataParser()
         ver = dc.version(od.RAW_DATA_OSTC_MK2_190)
         self.assertEquals('OSTC Mk.2 1.90', ver)
 
@@ -171,7 +187,7 @@ class MemoryDumpTestCase(unittest.TestCase):
         """
         Test OSTC N2 model and version parsing from raw data
         """
-        dc = OSTCMemoryDump()
+        dc = OSTCDataParser()
         ver = dc.version(od.RAW_DATA_OSTC_N2_191_HW)
         self.assertEquals('OSTC N2 1.91', ver)
 
@@ -180,7 +196,7 @@ class MemoryDumpTestCase(unittest.TestCase):
         """
         Test OSTC 1.91 and higher version parsing
         """
-        dc = OSTCMemoryDump()
+        dc = OSTCDataParser()
 
         ver = dc.version(od.RAW_DATA_OSTC_MK2_194)
         self.assertEquals('OSTC Mk.2 1.94', ver)
