@@ -136,12 +136,18 @@ class ParserTestCase(unittest.TestCase):
         self.assertEquals(275, header.min_temp)
         self.assertEquals(1025, header.surface_pressure)
         self.assertEquals(920, header.desaturation)
-        self.assertEquals(21, header.gas1)
-        self.assertEquals(32, header.gas2)
-        self.assertEquals(21, header.gas3)
-        self.assertEquals(21, header.gas4)
-        self.assertEquals(21, header.gas5)
-        self.assertEquals(32, header.gas6)
+        self.assertEquals(21, header.gas1_o2)
+        self.assertEquals(32, header.gas2_o2)
+        self.assertEquals(21, header.gas3_o2)
+        self.assertEquals(21, header.gas4_o2)
+        self.assertEquals(21, header.gas5_o2)
+        self.assertEquals(32, header.gas6_o2)
+        self.assertEquals(0, header.gas1_he)
+        self.assertEquals(0, header.gas2_he)
+        self.assertEquals(0, header.gas3_he)
+        self.assertEquals(0, header.gas4_he)
+        self.assertEquals(0, header.gas5_he)
+        self.assertEquals(0, header.gas6_he)
         self.assertEquals(1, header.gas)
         self.assertEquals(1, header.ver1)
         self.assertEquals(26, header.ver2)
@@ -176,12 +182,18 @@ class ParserTestCase(unittest.TestCase):
         self.assertEquals(57, header.min_temp)
         self.assertEquals(975, header.surface_pressure)
         self.assertEquals(248, header.desaturation)
-        self.assertEquals(21, header.gas1)
-        self.assertEquals(47, header.gas2)
-        self.assertEquals(100, header.gas3)
-        self.assertEquals(11532, header.gas4)
-        self.assertEquals(9489, header.gas5)
-        self.assertEquals(12810, header.gas6)
+        self.assertEquals(21, header.gas1_o2)
+        self.assertEquals(47, header.gas2_o2)
+        self.assertEquals(100, header.gas3_o2)
+        self.assertEquals(12, header.gas4_o2)
+        self.assertEquals(17, header.gas5_o2)
+        self.assertEquals(10, header.gas6_o2)
+        self.assertEquals(0, header.gas1_he)
+        self.assertEquals(0, header.gas2_he)
+        self.assertEquals(0, header.gas3_he)
+        self.assertEquals(45, header.gas4_he)
+        self.assertEquals(37, header.gas5_he)
+        self.assertEquals(50, header.gas6_he)
         self.assertEquals(4, header.gas)
         self.assertEquals(1, header.ver1)
         self.assertEquals(93, header.ver2)
