@@ -25,9 +25,11 @@ from lxml import etree as et
 from io import BytesIO
 from datetime import datetime
 from functools import partial
+from dirty.xml import xml
 import unittest
 
 import kenozooid.uddf as ku
+import kenozooid.data as kd
 
 
 UDDF_PROFILE = b"""\
@@ -198,6 +200,11 @@ UDDF_SITE = b"""\
 </uddf>
 """
 
+def xml2et(data):
+    doc = et.XML(''.join(l for l in data))
+    sd = et.tostring(doc, pretty_print=True)
+    return doc, sd
+
 
 class FindDataTestCase(unittest.TestCase):
     """
@@ -235,7 +242,7 @@ class FindDataTestCase(unittest.TestCase):
         
     def test_parsing(self):
         """
-        Test basic XML parsing routine.
+        Test basic XML parsing routine
         """
         f = BytesIO(UDDF_PROFILE)
         depths = list(ku.parse(f, '//uddf:waypoint//uddf:depth/text()'))
@@ -247,7 +254,7 @@ class FindDataTestCase(unittest.TestCase):
 
     def test_dive_data(self):
         """
-        Test parsing UDDF default dive data.
+        Test parsing UDDF default dive data
         """
         f = BytesIO(UDDF_PROFILE)
         node = next(ku.parse(f, '//uddf:dive[1]'))
@@ -260,22 +267,22 @@ class FindDataTestCase(unittest.TestCase):
 
     def test_profile_data(self):
         """
-        Test parsing UDDF default dive profile data.
+        Test parsing UDDF default dive profile data
         """
         f = BytesIO(UDDF_PROFILE)
         node = next(ku.parse(f, '//uddf:dive[2]'))
         profile = list(ku.dive_profile(node))
         self.assertEquals(4, len(profile))
 
-        self.assertEquals((0, 2.61, 296.73, None, None, None), profile[0])
-        self.assertEquals((10, 4.18, None, None, None, None), profile[1])
-        self.assertEquals((20, 6.25, None, None, None, None), profile[2])
-        self.assertEquals((30, 8.32, 297.26, None, None, None), profile[3])
+        self.assertEquals(kd.Sample(depth=2.61, time=0, temp=296.73), profile[0])
+        self.assertEquals(kd.Sample(depth=4.18, time=10), profile[1])
+        self.assertEquals(kd.Sample(depth=6.25, time=20), profile[2])
+        self.assertEquals(kd.Sample(depth=8.32, time=30, temp=297.26), profile[3])
 
 
     def test_dump_data(self):
         """
-        Test parsing UDDF dive computer dump data.
+        Test parsing UDDF dive computer dump data
         """
         f = BytesIO(UDDF_DUMP)
         node = next(ku.parse(f, '//uddf:divecomputerdump'))
@@ -290,7 +297,7 @@ class FindDataTestCase(unittest.TestCase):
 
     def test_dump_data_decode(self):
         """
-        Test dive computer data decoding stored in UDDF dive computer dump file.
+        Test dive computer data decoding stored in UDDF dive computer dump file
         """
         data = 'QlpoOTFBWSZTWZdWXlwAAAAJAH/gPwAgACKMmAAUwAE0xwH5Gis6xNXmi7kinChIS6svLgA='
         s = ku._dump_decode(data)
@@ -299,7 +306,7 @@ class FindDataTestCase(unittest.TestCase):
 
     def test_site_data(self):
         """
-        Test dive site data parsing.
+        Test dive site data parsing
         """
         f = BytesIO(UDDF_SITE)
         node = next(ku.parse(f, '//uddf:site[1]'))
@@ -310,7 +317,7 @@ class FindDataTestCase(unittest.TestCase):
 
     def test_buddy_query(self):
         """
-        Test buddy XPath query.
+        Test buddy XPath query
         """
         # by id and name
         self._qt(UDDF_BUDDY, ku.XP_FIND_BUDDY, 'b1', buddy='b1') # by id
@@ -326,7 +333,7 @@ class FindDataTestCase(unittest.TestCase):
 
     def test_site_query(self):
         """
-        Test dive site XPath query.
+        Test dive site XPath query
         """
         # by id
         self._qt(UDDF_SITE, ku.XP_FIND_SITE, 'konig', site='konig') 
@@ -345,7 +352,7 @@ class CreateDataTestCase(unittest.TestCase):
     """
     def test_create_basic(self):
         """
-        Test basic UDDF file creation.
+        Test basic UDDF file creation
         """
         now = datetime.now()
 
@@ -353,8 +360,8 @@ class CreateDataTestCase(unittest.TestCase):
         self.assertEquals('3.0.0', doc.get('version'))
 
         q = '//uddf:generator/uddf:datetime/text()'
-        dt = doc.xpath(q, namespaces=ku._NSMAP)
-        self.assertEquals(now.strftime(ku.FMT_DATETIME), dt[0])
+        dt = ku.xp_first(doc, q)
+        self.assertEquals(ku.FMT_DT(now), dt)
 
 
     def test_save(self):
@@ -681,74 +688,36 @@ class CreateDataTestCase(unittest.TestCase):
         """
         Test creating dive computer information data in UDDF file
         """
-        doc = ku.create()
-        xpath = partial(doc.xpath, namespaces=ku._NSMAP)
-        owner = xpath('//uddf:owner')[0]
+        n = ku.create_dc_data('tid', model='Test 1')
+        doc, sd = xml2et(ku.create_uddf(equipment=n))
 
-        ku.create_dc_data(owner, dc_model='Test 1')
-        sd = et.tostring(doc, pretty_print=True)
+        r = ku.xp_first(doc, '//uddf:owner//uddf:divecomputer/@id')
+        self.assertEquals(r, 'tid', sd)
 
-        id_q = '//uddf:owner//uddf:divecomputer/@id'
-        ids = xpath(id_q)
-        self.assertEquals(1, len(ids), sd)
-        self.assertEquals('id-206a9b642b3e16c89a61696ab28f3d5c', ids[0], sd)
-
-        model_q = '//uddf:owner//uddf:divecomputer/uddf:model/text()'
-        models = xpath(model_q)
-        self.assertEquals('Test 1', models[0], sd)
-
-        # update again with the same model
-        ku.create_dc_data(owner, dc_model='Test 1')
-        sd = et.tostring(doc, pretty_print=True)
-        ids = xpath(id_q)
-        self.assertEquals(1, len(ids), sd)
-
-        # add different model
-        ku.create_dc_data(owner, dc_model='Test 2')
-        sd = et.tostring(doc, pretty_print=True)
-
-        eqs = xpath('//uddf:equipment')
-        self.assertEquals(1, len(eqs), sd)
-
-        ids = xpath(id_q)
-        self.assertEquals(2, len(ids), sd)
-        expected = ['id-206a9b642b3e16c89a61696ab28f3d5c',
-                'id-605e79544a68819ce664c088aba92658']
-        self.assertEquals(expected, ids, sd)
-
-        models = xpath(model_q)
-        expected = ['Test 1', 'Test 2']
-        self.assertEquals(expected, models, sd)
+        r = ku.xp_first(doc, '//uddf:owner//uddf:divecomputer/uddf:model/text()')
+        self.assertEquals(r, 'Test 1', sd)
 
 
-    def test_create_dive_profile_sample_default(self):
+    def test_create_dive_samples(self):
         """
-        Test UDDF dive profile default sample creation
+        Test UDDF dive profile sample creation
         """
-        w = ku.create_dive_profile_sample(None, depth=3.1, time=19, temp=20)
-        s = et.tostring(w)
-        self.assertEquals('19', ku.xp_first(w, 'uddf:divetime/text()'), s)
-        self.assertEquals('3.1', ku.xp_first(w, 'uddf:depth/text()'), s)
-        self.assertEquals('20.0', ku.xp_first(w, 'uddf:temperature/text()'), s)
+        s = kd.Sample(depth=3.1, time=19, temp=20),
+        samples = ku.create_dive_samples(s)
+        n, sd = xml2et(xml.dive(samples))
+        self.assertEquals('19', ku.xp_first(n, '//divetime/text()'), sd)
+        self.assertEquals('3.1', ku.xp_first(n, '//depth/text()'), sd)
+        self.assertEquals('20.0', ku.xp_first(n, '//temperature/text()'), sd)
 
 
-    def test_create_dive_profile_sample_custom(self):
+    def test_create_dive_samples(self):
         """
-        Test UDDF dive profile custom sample creation
+        Test UDDF dive profile sample creation with deco alarm
         """
-        Q = {
-            'depth': 'uddf:depth',
-            'time': 'uddf:divetime',
-            'temp': 'uddf:temperature',
-            'alarm': 'uddf:alarm',
-        }
-        w = ku.create_dive_profile_sample(None, queries=Q,
-                depth=3.1, time=19, temp=20, alarm='deco')
-        s = et.tostring(w)
-        self.assertEquals('19', ku.xp_first(w, 'uddf:divetime/text()'), s)
-        self.assertEquals('3.1', ku.xp_first(w, 'uddf:depth/text()'), s)
-        self.assertEquals('20.0', ku.xp_first(w, 'uddf:temperature/text()'), s)
-        self.assertEquals('deco', ku.xp_first(w, 'uddf:alarm/text()'), s)
+        s = kd.Sample(depth=3.1, time=19, temp=20, alarm=('deco',)),
+        samples = ku.create_dive_samples(s)
+        n, sd = xml2et(xml.dive(samples))
+        self.assertEquals('deco', ku.xp_first(n, '//alarm/text()'), sd)
         
 
     def test_dump_data_encode(self):
@@ -794,7 +763,7 @@ class CreateDataTestCase(unittest.TestCase):
 
     def test_create_site_with_pos(self):
         """
-        Test creating dive site data with position.
+        Test creating dive site data with position
         """
         f = ku.create()
         buddy = ku.create_site_data(f, name='SMS Konig', location='Scapa Flow',
@@ -809,7 +778,7 @@ class CreateDataTestCase(unittest.TestCase):
 
     def test_create_site_no_id(self):
         """
-        Test creating dive site data with autogenerated id.
+        Test creating dive site data with autogenerated id
         """
         f = ku.create()
         buddy = ku.create_site_data(f, name='Konig', location='Scapa Flow')
@@ -1320,7 +1289,7 @@ class NodeCopyTestCase(unittest.TestCase):
 
     def test_error_on_empty_copy(self):
         """
-        Test error on empty copy.
+        Test error on empty copy
         """
         SOURCE = b"""\
 <?xml version="1.0" encoding="utf-8"?>
