@@ -25,18 +25,16 @@ The functions implemented in this module can be divided into the following
 categories
 
 - XML nodes functions
-- generic data searching and manipulation functions
+- generic XML data searching and manipulation functions
 - functions for searching and manipulation of diving specific data
 
-Almost each function accepts XPath expression (query) to find or modify
-data. Each tag name in an query should be prefixed with 'uddf:' string to
-indicate UDDF namespace, i.e. 'uddf:diver', 'uddf:waypoint' - appropriate
-namespace mapping for this prefix is defined for each XPath call or during
-XML node creation.
+Searching functions use XPath expressions (queries) to find data. Each tag
+name in an query should be prefixed with 'uddf:' string to indicate UDDF
+namespace, i.e. 'uddf:diver', 'uddf:waypoint' - appropriate namespace
+mapping for this prefix is defined for each XPath call.
 
 The result of parsing or search of data is usually iterator of XML nodes or
-data records (named tuples in Python terms) depending on type of function
-(see categories above).
+data records (named tuples in Python terms).
 
 Module `lxml` is used for XML parsing and querying with XPath. Full
 capabilities of underlying `libxml2' library is used by design. The
@@ -51,13 +49,17 @@ from dateutil.parser import parse as dparse
 from operator import itemgetter
 from uuid import uuid4 as uuid
 from copy import deepcopy
+from dirty.xml import xml
+from dirty import RawString
 import base64
 import bz2
+import itertools
 import hashlib
 import logging
 import pkg_resources
 
 import kenozooid
+import kenozooid.util as kt
 
 log = logging.getLogger('kenozooid.uddf')
 
@@ -69,6 +71,10 @@ _NSMAP = {'uddf': 'http://www.streit.cc/uddf/3.0/'}
 # Node id formatter
 FORMAT_ID = 'id-{}'
 
+FMT_F = partial(str.format, '{0:.1f}')
+FMT_I = lambda v: '{}'.format(int(round(v)))
+FMT_DT = lambda dt: format(dt, '%Y-%m-%dT%H:%M:%S%z')
+
 #
 # Parsing and searching.
 #
@@ -77,45 +83,64 @@ FORMAT_ID = 'id-{}'
 XPath = partial(et.XPath, namespaces=_NSMAP)
 
 # XPath queries for default dive data
-XP_DEFAULT_DIVE_DATA = (XPath('uddf:informationbeforedive/uddf:datetime/text()'),
+XP_DEFAULT_DIVE_DATA = (
+    XPath('uddf:informationbeforedive/uddf:datetime/text()'),
     XPath('uddf:informationafterdive/uddf:greatestdepth/text()'),
     XPath('uddf:informationafterdive/uddf:diveduration/text()'),
     XPath('uddf:informationafterdive/uddf:lowesttemperature/text()'))
 
 # XPath queries for default dive profile sample data
-XP_DEFAULT_PROFILE_DATA =  (XPath('uddf:divetime/text()'),
-        XPath('uddf:depth/text()'),
-        XPath('uddf:temperature/text()'),
-        XPath('uddf:decostop/@duration'),
-        XPath('uddf:decostop/@decodepth'),
-        XPath('uddf:alarm/text()'))
+XP_DEFAULT_PROFILE_DATA =  (
+    XPath('uddf:depth/text()'),
+    XPath('uddf:divetime/text()'),
+    XPath('uddf:temperature/text()'),
+    XPath('uddf:decostop/@duration'),
+    XPath('uddf:decostop/@decodepth'),
+    XPath('uddf:alarm/text()'),
+    XPath('uddf:switchmix/@ref'),
+)
+
+XP_DEFAULT_GAS_DATA =  (
+    XPath('@id'),
+    XPath('uddf:name/text()'),
+    XPath('uddf:o2/text()'),
+    XPath('uddf:he/text()'),
+)
 
 # XPath query to locate dive profile sample
 XP_WAYPOINT = XPath('.//uddf:waypoint')
+# XPath query to locate gas mix
+XP_MIX = XPath('/uddf:uddf/uddf:gasdefinitions/uddf:mix')
 
 # XPath queries for default dive computer dump data
-XP_DEFAULT_DUMP_DATA = (XPath('uddf:link/@ref'),
-        # //uddf:divecomputerdump[position()] gives current()
-        XPath('../../uddf:diver/uddf:owner//uddf:divecomputer[' \
-                '@id = //uddf:divecomputerdump[position()]/uddf:link/@ref' \
-            ']/uddf:model/text()'),
-        XPath('uddf:datetime/text()'),
-        XPath('uddf:dcdump/text()'))
+XP_DEFAULT_DUMP_DATA = (
+    XPath('uddf:link/@ref'),
+    # //uddf:divecomputerdump[position()] gives current()
+    XPath('../../uddf:diver/uddf:owner//uddf:divecomputer[' \
+            '@id = //uddf:divecomputerdump[position()]/uddf:link/@ref' \
+        ']/uddf:model/text()'),
+    XPath('uddf:datetime/text()'),
+    XPath('uddf:dcdump/text()'),
+)
 
 # XPath queries for default buddy data
-XP_DEFAULT_BUDDY_DATA = (XPath('@id'),
-        XPath('uddf:personal/uddf:firstname/text()'),
-        XPath('uddf:personal/uddf:middlename/text()'),
-        XPath('uddf:personal/uddf:lastname/text()'),
-        XPath('uddf:personal/uddf:membership/@organisation'),
-        XPath('uddf:personal/uddf:membership/@memberid'))
+XP_DEFAULT_BUDDY_DATA = (
+    XPath('@id'),
+    XPath('uddf:personal/uddf:firstname/text()'),
+    XPath('uddf:personal/uddf:middlename/text()'),
+    XPath('uddf:personal/uddf:lastname/text()'),
+    XPath('uddf:personal/uddf:membership/@organisation'),
+    XPath('uddf:personal/uddf:membership/@memberid'),
+)
 
 # XPath queries for default dive site data
-XP_DEFAULT_SITE_DATA = (XPath('@id'),
-        XPath('uddf:name/text()'),
-        XPath('uddf:geography/uddf:location/text()'),
-        XPath('uddf:geography/uddf:longitude/text()'),
-        XPath('uddf:geography/uddf:latitude/text()'))
+XP_DEFAULT_SITE_DATA = (
+    XPath('@id'),
+    XPath('uddf:name/text()'),
+    XPath('uddf:geography/uddf:location/text()'),
+    XPath('uddf:geography/uddf:longitude/text()'),
+    XPath('uddf:geography/uddf:latitude/text()'),
+)
 
 # XPath query to find a buddy
 XP_FIND_BUDDY = XPath('/uddf:uddf/uddf:diver/uddf:buddy[' \
@@ -341,14 +366,26 @@ def dive_profile(node, fields=None, queries=None, parsers=None):
         find_data
     """
     if fields is None:
-        fields = ('time', 'depth', 'temp', 'deco_time', 'deco_depth', 'alarm')
+        fields = ('depth', 'time', 'temp', 'deco_time', 'deco_depth',
+                'alarm', 'gas')
         queries = XP_DEFAULT_PROFILE_DATA
-        parsers = (float, ) * 5 + (str, )
+        gases = dict(((gas.id, gas) for gas in gas_data(node)))
+        parsers = (float, ) * 5 + (str, gases.get)
 
     return find_data('Sample', node, fields, queries, parsers,
             nquery=XP_WAYPOINT)
 
 
+def gas_data(node, fields=None, queries=None, parsers=None):
+    if fields is None:
+        fields = ('id', 'name', 'o2', 'he')
+        queries = XP_DEFAULT_GAS_DATA
+        parsers = (str, str, int, int)
+
+    return find_data('Gas', node, fields, queries, parsers,
+            nquery=XP_MIX)
+
+
 def dump_data(node, fields=None, queries=None, parsers=None):
     """
     Get dive computer dump data.
@@ -559,15 +596,13 @@ def _dump_decode(data):
 # Creating UDDF data.
 #
 
-# default format for timestamps within UDDF file
-FMT_DATETIME = '%Y-%m-%dT%H:%M:%S%z'
-
 DEFAULT_FMT_DIVE_PROFILE = {
     'depth': lambda d: str.format('{0:.1f}', max(d, 0)),
     'temp': partial(str.format, '{0:.1f}'),
 }
 
 # basic data for an UDDF file
+# fixme: obsolete
 UDDF_BASIC = """\
 <uddf xmlns="http://www.streit.cc/uddf/3.0/" version="3.0.0">
 <generator>
@@ -592,15 +627,11 @@ UDDF_BASIC = """\
 </uddf>
 """.format(kzver=kenozooid.__version__)
 
-###<equipment>
-###    <divecomputer id=''>
-###        <model></model>
-###    </divecomputer>
-###</equipment>
-
 
 def create(datetime=datetime.now()):
     """
+    fixme: obsolete
+
     Create basic UDDF structure.
 
     :Parameters:
@@ -611,12 +642,14 @@ def create(datetime=datetime.now()):
 
     now = datetime.now()
     n = root.xpath('//uddf:generator/uddf:datetime', namespaces=_NSMAP)[0]
-    n.text = _format_time(datetime)
+    n.text = FMT_DT(datetime)
     return root
 
 
 def save(doc, f, validate=True):
     """
+    fixme: obsolete
+
     Save UDDF data to a file.
 
     A file can be a file name, file like object or anything supported by
@@ -758,44 +791,6 @@ def create_node(path, parent=None, append=True):
         yield n
 
 
-def create_dc_data(node, queries=None, formatters=None,
-        dc_id=None, dc_model=None, **data):
-    """
-    Create dive computer information data in UDDF file.
-    """
-    _f = ('dc_id', 'dc_model')
-    _q = ('@id', 'uddf:model')
-    _queries = OrderedDict(zip(_f, _q))
-    if queries is not None:
-        _queries.update(queries)
-
-    data['dc_id'] = dc_id
-    data['dc_model'] = dc_model
-
-    dc = None
-
-    if dc_id:
-        xp = XPath('uddf:equipment/uddf:divecomputer[id@$dc_id]')
-        nodes = xp(node, dc_id=dc_id)
-        if nodes:
-            dc = nodes[0]
-    else:
-        xp = XPath('uddf:equipment/uddf:divecomputer[uddf:model/text() = $dc_model]')
-        nodes = xp(node, dc_model=dc_model)
-        if nodes:
-            dc = nodes[0]
-
-    if dc is None:
-        if not dc_id:
-            dc_id = FORMAT_ID.format(hashlib.md5(dc_model.encode()).hexdigest())
-            data['dc_id'] = dc_id
-
-        # create new dive computer node
-        _, dc = create_node('uddf:equipment/uddf:divecomputer', parent=node)
-        set_data(dc, _queries, formatters, **data)
-    return dc
-
-
 def create_dive_data(node=None, queries=None, formatters=None, **data):
     """
     Create dive data.
@@ -823,7 +818,7 @@ def create_dive_data(node=None, queries=None, formatters=None, **data):
         queries = OrderedDict(zip(f, q))
     if formatters == None:
         formatters = {
-            'datetime': _format_time,
+            'datetime': FMT_DT,
             'depth': partial(str.format, '{0:.1f}'),
             'duration': partial(str.format, '{0:.0f}'),
             'temp': partial(str.format, '{0:.1f}'),
@@ -836,37 +831,6 @@ def create_dive_data(node=None, queries=None, formatters=None, **data):
     return dn
 
 
-def create_dive_profile_sample(node, queries=None, formatters=None, **data):
-    if queries == None:
-        f = ('depth', 'time', 'temp')
-        q = ('uddf:depth', 'uddf:divetime', 'uddf:temperature')
-        queries = OrderedDict(zip(f, q))
-    if formatters == None:
-        formatters = DEFAULT_FMT_DIVE_PROFILE
-
-    _, wn = create_node('uddf:samples/uddf:waypoint', parent=node)
-    set_data(wn, queries, formatters, **data)
-    return wn
-
-
-def create_dump_data(node, queries=None, formatters=None, **data):
-    if queries == None:
-        f = ('dc_id', 'datetime', 'data')
-        q = ('uddf:link/@ref', 'uddf:datetime', 'uddf:dcdump')
-        queries = OrderedDict(zip(f, q))
-
-    if formatters == None:
-        formatters = {
-            'datetime': _format_time,
-            'data': _dump_encode,
-        }
-        
-    _, dcd = create_node('uddf:divecomputercontrol/uddf:divecomputerdump',
-            parent=node)
-    set_data(dcd, queries, formatters, **data)
-    return dcd
-
-
 def create_buddy_data(node, queries=None, formatters=None, **data):
     """
     Create buddy data.
@@ -938,13 +902,6 @@ def create_site_data(node, queries=None, formatters=None, **data):
     return site
         
 
-def _format_time(t):
-    """
-    Format timestamp into ISO 8601 string compatible with UDDF.
-    """
-    return format(t, FMT_DATETIME)
-
-
 def _dump_encode(data):
     """
     Encode dive computer data, so it can be stored in UDDF file.
@@ -955,6 +912,191 @@ def _dump_encode(data):
     return base64.b64encode(s)
 
 
+def create_uddf(datetime=datetime.now(), equipment=None, gases=None, dives=None,
+        dump=None):
+    """
+    Create UDDF XML data.
+
+    :Parameters:
+     datetime
+        Timestamp of UDDF creation.
+     equipment
+        Diver's (owner) equipment XML data (see create_dc_data).
+     gases
+        List of gases used by the dives.
+     dives
+        Dives XML data (see create_dives).
+     dump
+        Dive computer dump XML data (see create_dump_data).
+    """
+    doc = xml.uddf(
+        xml.generator(
+            xml.name('kenozooid'),
+            xml.manufacturer(
+                xml.name('Kenozooid Team'),
+                xml.contact(xml.homepage('http://wrobell.it-zone.org/kenozooid/')),
+                id='kenozooid'),
+            xml.version(kenozooid.__version__),
+            xml.datetime(FMT_DT(datetime)),
+        ),
+
+        xml.diver(
+            xml.owner(
+                xml.personal(xml.firstname('Anonymous'), xml.lastname('Guest')),
+                xml.equipment(equipment) if equipment else None,
+                id='owner')),
+
+        xml.divecomputercontrol(dump) if dump else None,
+        xml.gasdefinitions(gases) if gases else None,
+        xml.profiledata(xml.repetitiongroup(dives, id=gen_id()))
+            if dives else None,
+
+        xmlns=_NSMAP['uddf'],
+        version='3.0.0',
+    )
+    return doc
+
+
+def create_dives(dives, equipment=None):
+    """
+    Create dives UDDF XML data.
+
+    :Parameters:
+     dives
+        Iterable of dive tuples.
+     equipment
+        List of used equipment references.
+    """
+    for dive in dives:
+        log.debug('convert dive {0.datetime}/{0.depth:.1f}/{0.duration} into XML'
+                .format(dive))
+        yield create_dive(dive, equipment)
+
+
+def create_dive(dive, equipment=None):
+    """
+    Create dive UDDF XML data.
+
+    :Parameters:
+     dive
+        Dive to render as XML.
+     equipment
+        List of used equipment references.
+    """
+    eq = itertools.chain(kt.nit(dive.equipment), kt.nit(equipment))
+    log.debug('convert dive {0.datetime}/{0.depth:.1f}/{0.duration} into XML'
+            .format(dive))
+    return xml.dive(
+        xml.informationbeforedive(xml.datetime(FMT_DT(dive.datetime))),
+        xml.samples(create_dive_samples(dive.profile)),
+        xml.informationafterdive(
+            xml.greatestdepth(FMT_F(dive.depth)),
+            xml.diveduration(FMT_I(dive.duration)),
+            None if dive.temp is None else xml.lowesttemperature(FMT_F(dive.temp)),
+            xml.equipmentused((xml.link(ref=v) for v in eq)),
+        ),
+        id=gen_id(),
+    )
+
+
+def create_dive_samples(samples):
+    """
+    Create dive samples UDDF XML data.
+
+    :Parameters:
+     samples
+        Iterable of tuples of dive samples.
+    """
+    for s in samples:
+        yield xml.waypoint(
+            None if s.alarm is None else (xml.alarm(a) for a in s.alarm),
+            None if s.deco_time is None else
+                xml.decostop(
+                    duration=FMT_I(s.deco_time),
+                    decodepth=FMT_I(s.deco_depth),
+                    kind='mandatory'
+                ),
+            xml.depth(FMT_F(s.depth)),
+            xml.divetime(FMT_I(s.time)),
+            None if s.gas is None else xml.switchmix(ref=str(s.gas.id)),
+            None if s.temp is None else xml.temperature(FMT_F(s.temp)),
+        )
+
+
+def create_gas(gas):
+    """
+    Create gas UDDF XML data.
+
+    :Parameters:
+     gas
+        Gas information to render as XML.
+    """
+    return xml.mix(
+        xml.name(gas.name),
+        xml.o2(str(gas.o2)),
+        xml.he(str(gas.he)),
+        id=gas.id,
+    )
+
+
+def create_dc_data(dc_id, model):
+    """
+    Create dive computer UDDF XML data.
+
+    :Parameters:
+     dc_id
+        Dive computer id.
+     model
+        Dive computer model.
+    """
+    yield xml.divecomputer(xml.model(model), id=dc_id)
+
+
+def create_dump_data(dc_id, datetime, data):
+    """
+    Create dive computer dump UDDF XML data.
+
+    :Parameters:
+     dc_id
+        Dive computer id.
+     datetime
+        Date and time when the dump was created.
+     data
+        Dive computer binary data.
+    """
+    yield xml.divecomputerdump(
+        xml.link(ref=dc_id),
+        xml.datetime(FMT_DT(datetime)),
+        xml.dcdump(_dump_encode(data).decode()),
+    )
+
+
+def save_uddf(doc, fout, validate=True):
+    """
+    Save UDDF XML data into a file.
+
+    :Parameters:
+     doc
+        UDDF XML data.
+     fout
+        Output file.
+     validate
+        Validate UDDF file after saving if True.
+    """
+    log.debug('saving uddf file')
+    with open(fout, 'w') as f:
+        f.writelines(doc)
+
+    if validate:
+        log.debug('validating uddf file')
+        fs = pkg_resources.resource_stream('kenozooid', 'uddf_3.0.1.xsd')
+        if hasattr(fs, 'name'):
+            log.debug('uddf xsd found: {}'.format(fs.name))
+        schema = et.XMLSchema(et.parse(fs))
+        schema.assertValid(et.parse(fout))
+        log.debug('uddf file is valid')
+
+
 #
 # Removing UDDF data.
 #
@@ -983,6 +1125,8 @@ def remove_nodes(node, query, **params):
 
 def reorder(doc):
     """
+    fixme: obsolete
+
     Reorder and cleanup dives in UDDF document.
 
     Following operations are being performed
@@ -993,8 +1137,6 @@ def reorder(doc):
     :Parameters:
      doc
         UDDF document.
-
-    TODO: Put dives into appropriate repetition groups.
     """
     find = partial(doc.xpath, namespaces=_NSMAP)
 
@@ -1090,4 +1232,36 @@ def _set_id(node):
         node.set('id', FORMAT_ID.format(uuid().hex))
 
 
+def gen_id(value=None):
+    """
+    Generate id for a value.
+    
+    If value is specified then id is MD5 hash of value. If not specified,
+    then id is generated with UUID 4.
+
+    The returned id is a string prefixed with ``id-`` to make it XML
+    compliant.
+    """
+    if value is None:
+        vid = uuid().hex
+    else:
+        vid = hashlib.md5(str(value).encode()).hexdigest()
+    return 'id-{}'.format(vid)
+
+
+def xml_file_copy(f):
+    """
+    Iterator of raw XML data from a file to data into ``dirty.xml`` nodes.
+
+    :Parameters:
+     f
+        File containing XML data.
+    """
+    while True:
+        l = f.read(4096)
+        if not l:
+            break
+        yield RawString(l)
+
+
 # vim: sw=4:et:ai
